@@ -1,7 +1,7 @@
 # +--------------------------------------------------------------------------+
 # |  Licensed Materials - Property of IBM                                    |
 # |                                                                          |
-# | (C) Copyright IBM Corporation 2009.                                      |
+# | (C) Copyright IBM Corporation 2009-2013.                                      |
 # +--------------------------------------------------------------------------+
 # | This module complies with Django 1.0 and is                              |
 # | Licensed under the Apache License, Version 2.0 (the "License");          |
@@ -17,7 +17,13 @@
 # +--------------------------------------------------------------------------+
 
 from django.db.models.sql import compiler
-
+import sys
+if sys.version_info >= (3, ):
+    try:
+        from itertools import zip_longest
+    except ImportError:
+        from itertools import izip_longest as zip_longest
+        
 class SQLCompiler( compiler.SQLCompiler ):
     __rownum = 'Z.__ROWNUM'
 
@@ -90,11 +96,17 @@ class SQLCompiler( compiler.SQLCompiler ):
 
         return sql, params
     
+    def __map23(self, value, field):
+        if sys.version_info >= (3, ):
+            return zip_longest(value, field)
+        else:
+            return map(None, value, field)
+        
     #This function  convert 0/1 to boolean type for BooleanField/NullBooleanField
     def resolve_columns( self, row, fields = () ):
         values = []
         index_extra_select = len( self.query.extra_select.keys() )
-        for value, field in map( None, row[index_extra_select:], fields ):
+        for value, field in self.__map23( row[index_extra_select:], fields ):
             if ( field and field.get_internal_type() in ( "BooleanField", "NullBooleanField" ) and value in ( 0, 1 ) ):
                 value = bool( value )
             values.append( value )
